import React from 'react';
import Svg, { Line } from 'react-native-svg';
import { Dimensions, View } from 'react-native';
import _ from 'lodash';
import moment from 'moment';
import { Dispatcher } from '../../globals/Dispatcher';
import HttpController from '../../controllers/HttpController';

type Props = {
  audio_recorder: {},
};

const SAMPLE_RATE = 16000;
const TIMEFRAME_LEN_SEC = 6;
const SAMPLE_SPACING = 1000; // >= 1 | Low values can and will kill app
const UPDATE_THROTTLE_MS = 200; // At least 100 needed for render
const UX_WAVE_SCALE = 500;

const MAX_WAVE_DEFAULT = 500;
const MAX_WAVE_DECAY_IN_SEC = 3000;

export default class ComponentAudioWaveform extends React.Component<Props> {
  constructor(props) {
    try {
      super(props);
      this.color = '#D0D1F7';
      this.height = 100;
      this.width = Dimensions.get('screen').width;
      this.state = {
        waves: Array(SAMPLE_RATE * TIMEFRAME_LEN_SEC).fill(0),
      };
      this.throttleState = _.throttle(function () {
        try {
          if (this.is_mounted) {
            this.setState(this.state);
          }
        } catch (exc) {
          HttpController.logException(
            'ComponentAudioWaveform.constructor',
            exc,
          );
        }
      }, UPDATE_THROTTLE_MS);
      this.listener = null;
      this.ticker = null;
      this.is_mounted = true;
      this.listener = Dispatcher.addListener('AUDIO_CHUNK', this.onAudioChunk);

      this.max_wave = MAX_WAVE_DEFAULT;

      this.debug_moment = moment();
      this.moment_max_wave_decay = moment();
    } catch (exc) {
      HttpController.logException('ComponentAudioWaveform.constructor', exc);
    }
  }

  shouldComponentUpdate(
    nextProps: Readonly<Props>,
    nextState: Readonly<State>,
    nextContext: any,
  ): boolean {
    this.color = nextProps.line_color ? nextProps.line_color : '#000';
    return true;
  }

  onAudioChunk = async (data) => {
    try {
      let new_arr = new Int16Array(data.buffer);
      new_arr = Array.from(new_arr.values());
      // console.log(new_arr);

      let newWave = this.state.waves;
      newWave = newWave.concat(new_arr);

      if (newWave.length > SAMPLE_RATE * TIMEFRAME_LEN_SEC) {
        newWave.splice(0, newWave.length - SAMPLE_RATE * TIMEFRAME_LEN_SEC); // Limit length
      }

      this.state.waves = newWave;
      this.throttleState();
    } catch (e) {}
  };

  componentWillUnmount = () => {
    try {
      this.is_mounted = false;
      if (this.props.audio_recorder && this.listener) {
        this.listener.remove();
      }
      if (this.ticker) {
        clearTimeout(this.ticker);
      }
    } catch (e) {}
  };

  render = () => {
    try {
      let lines = [];

      this.max_wave -=
        (MAX_WAVE_DECAY_IN_SEC * moment().diff(this.moment_max_wave_decay)) /
        1000;
      this.moment_max_wave_decay = moment();

      this.max_wave = Math.max(
        MAX_WAVE_DEFAULT,
        this.max_wave,
        _.max(this.state.waves.map((it) => Math.abs(it))),
      );

      for (
        let l = 0;
        l < SAMPLE_RATE * TIMEFRAME_LEN_SEC;
        l += SAMPLE_SPACING
      ) {
        let sample_val = _.max(
          this.state.waves
            .slice(l, l + SAMPLE_SPACING)
            .map((it) => Math.abs(it)),
        );
        let sample_val_scaled = (UX_WAVE_SCALE * sample_val) / this.max_wave;
        let lineStart = UX_WAVE_SCALE - sample_val_scaled;
        let lineEnd = 2 * UX_WAVE_SCALE - lineStart;
        lines.push(
          <Line
            x1={l / SAMPLE_SPACING}
            x2={l / SAMPLE_SPACING}
            y1={lineStart}
            y2={lineEnd}
            stroke={this.color}
            strokeWidth={1}
            strokeLinecap={'round'}
            key={l}
          />,
        );
      }
      return (
        <View
          style={{
            width: this.width,
            height: this.height,
            ...(this.props.containerStyle ? this.props.containerStyle : {}),
          }}>
          <Svg
            height="100%"
            width="100%"
            viewBox={`0 0 ${
              (SAMPLE_RATE * TIMEFRAME_LEN_SEC) / SAMPLE_SPACING
            } 1000`}
            preserveAspectRatio="none">
            {lines}
          </Svg>
        </View>
      );
    } catch (e) {}
    return false;
  };
}
